#!/bin/sh
#
# JUST Firewall script
# Author support@just.social
#
###############################################################################

help () {
	echo "Usage: $0 (save|restore|stop) (--vpn-enabled) (--internet-iface XXX)"
	echo "e.g $0 --vpn-enabled --interface-iface eth0"
	exit 0
}

checkInterface() {
    if [ "$1" = "any" ]; then
        found="true"
    else
        found="false"
    fi
    if [ "$found" = "false" ]
    then
        echo "Interface $1 does not exist. Exiting."
        exit 1
    fi
}

# Check the parameter

#VPN enabled
VPN_ENABLED="false"

# Internet Interface
INET_IFACE="eth0"
# Indicate if we want to shutdown the firewall
STOP="false"

while [ $# -ne 0 ]
do
    case "$1" in
        save)
            # Save and Restore arguments handled here
            echo -n "Saving firewall to /etc/sysconfig/iptables ... "
            $IPTS > /etc/sysconfig/iptables
            echo "done"
            exit 0
            ;;
        restore)
	        echo -n "Restoring firewall from /etc/sysconfig/iptables ... "
	        $IPTR < /etc/sysconfig/iptables
	    	echo "done"
	    	exit 0
            ;;
        stop)
            STOP="true"
            ;;
        --vpn-enabled)
            # Initialise the vpn settings
            VPN_ENABLED="true"
            ;;
        --internet-iface)
            # Initialise the interface interface (default is eth0)
            INET_IFACE=$2
            shift
            ;;
        *)
            help
            ;;
    esac
    shift
done

checkInterface $INET_IFACE

if [ "$INET_IFACE" = "any" ]; then
    INET_IFACE=""
    INET_IFACE_O=""
else
    INET_IFACE_O="-o $INET_IFACE"
    INET_IFACE="-i $INET_IFACE"
fi
echo "Starting script as $0 --vpn-enabled $VPN_ENABLED --internet-iface $INET_IFACE"

#
# Local Settings
#

SYSCTL="/sbin/sysctl -w"
IPT=`which iptables`
IPTS=`which iptables-save`
IPTR=`which iptables-restore`
LO_IFACE="lo"
LO_IP="127.0.0.1"


###############################################################################
#
# Load Modules
#

echo "Loading kernel modules ..."

# Unless you have kernel module auto-loading disabled, you should not
# need to manually load each of these modules.

# core netfilter module
/sbin/modprobe ip_tables

# the stateful connection tracking module
/sbin/modprobe ip_conntrack

# filter table module
# /sbin/modprobe iptable_filter

# mangle table module
# /sbin/modprobe iptable_mangle

# nat table module
# /sbin/modprobe iptable_nat

# LOG target module
# /sbin/modprobe ipt_LOG

# This is used to limit the number of packets per sec/min/hr
# /sbin/modprobe ipt_limit

# masquerade target module
# /sbin/modprobe ipt_MASQUERADE

# filter using owner as part of the match
# /sbin/modprobe ipt_owner

# REJECT target drops the packet and returns an ICMP response.
# The response is configurable.  By default, connection refused.
# /sbin/modprobe ipt_REJECT

# This target allows packets to be marked in the mangle table
# /sbin/modprobe ipt_mark

# This target affects the TCP MSS
# /sbin/modprobe ipt_tcpmss

# This match allows multiple ports instead of a single port or range
# /sbin/modprobe multiport

# This match checks against the TCP flags
# /sbin/modprobe ipt_state

# This match catches packets with invalid flags
# /sbin/modprobe ipt_unclean

# The ftp nat module is required for non-PASV ftp support
/sbin/modprobe ip_nat_ftp

# the module for full ftp connection tracking
/sbin/modprobe ip_conntrack_ftp

# the module for full irc connection tracking
/sbin/modprobe ip_conntrack_irc


###############################################################################
#
# Kernel Parameter Configuration
#

# This enables SYN flood protection.
# The SYN cookies activation allows your system to accept an unlimited
# number of TCP connections while still trying to give reasonable
# service during a denial of service attack.
if [ "$SYSCTL" = "" ]
then
    echo "1" > /proc/sys/net/ipv4/tcp_syncookies
velse
    $SYSCTL net.ipv4.tcp_syncookies="1"
fi

# This enables source validation by reversed path according to RFC1812.
# In other words, did the response packet originate from the same interface
# through which the source packet was sent?  It's recommended for single-homed
# systems and routers on stub networks.  Since those are the configurations
# this firewall is designed to support, I turn it on by default.
# Turn it off if you use multiple NICs connected to the same network.
if [ "$SYSCTL" = "" ]
then
    echo "1" > /proc/sys/net/ipv4/conf/all/rp_filter
else
    $SYSCTL net.ipv4.conf.all.rp_filter="1"
fi

# This kernel parameter instructs the kernel to ignore all ICMP
# echo requests sent to the broadcast address.  This prevents
# a number of smurfs and similar DoS nasty attacks.
if [ "$SYSCTL" = "" ]
then
    echo "1" > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts
else
    $SYSCTL net.ipv4.icmp_echo_ignore_broadcasts="1"
fi

# This option can be used to accept or refuse source routed
# packets.  It is usually on by default, but is generally
# considered a security risk.  This option turns it off.
if [ "$SYSCTL" = "" ]
then
    echo "0" > /proc/sys/net/ipv4/conf/all/accept_source_route
else
    $SYSCTL net.ipv4.conf.all.accept_source_route="0"
fi

# However, we'll ensure the secure_redirects option is on instead.
# This option accepts only from gateways in the default gateways list.
if [ "$SYSCTL" = "" ]
then
    echo "1" > /proc/sys/net/ipv4/conf/all/secure_redirects
else
    $SYSCTL net.ipv4.conf.all.secure_redirects="1"
fi

# This option logs packets from impossible addresses.
if [ "$SYSCTL" = "" ]
then
    echo "1" > /proc/sys/net/ipv4/conf/all/log_martians
else
    $SYSCTL net.ipv4.conf.all.log_martians="1"
fi


###############################################################################
#
# Flush Any Existing Rules or Chains
#

echo "Flushing Tables ..."

# Reset Default Policies
$IPT -P INPUT ACCEPT
$IPT -P FORWARD ACCEPT
$IPT -P OUTPUT ACCEPT
$IPT -t nat -P PREROUTING ACCEPT
$IPT -t nat -P POSTROUTING ACCEPT
$IPT -t nat -P OUTPUT ACCEPT
$IPT -t mangle -P PREROUTING ACCEPT
$IPT -t mangle -P OUTPUT ACCEPT

# Flush all rules
$IPT -F
$IPT -t nat -F
$IPT -t mangle -F

# Erase all non-default chains
$IPT -X
$IPT -t nat -X
$IPT -t mangle -X

if [ "$STOP" = "true" ]
then
	echo "Firewall completely flushed!  Now running with no firewall."
	exit 0
fi

###############################################################################
#
# Rules Configuration
#

###############################################################################
#
# Filter Table
#
###############################################################################

# Set Policies

$IPT -P INPUT DROP
$IPT -P OUTPUT DROP
$IPT -P FORWARD DROP

###############################################################################
#
# User-Specified Chains
#
# Create user chains to reduce the number of rules each packet
# must traverse.

echo "Create and populate custom rule chains ..."

# Create a chain to filter INVALID packets

$IPT -N bad_packets

# Create another chain to filter bad tcp packets

$IPT -N bad_tcp_packets

# Create separate chains for icmp, tcp (incoming and outgoing),
# and incoming udp packets.

$IPT -N icmp_packets

# Used for UDP packets inbound from the Internet
$IPT -N udp_inbound

# Used to block outbound UDP services from internal network
# Default to allow all
$IPT -N udp_outbound

# Used to allow inbound services if desired
# Default fail except for established sessions
$IPT -N tcp_inbound

# Used to block outbound services from internal network
# Default to allow all
$IPT -N tcp_outbound

###############################################################################
#
# Populate User Chains
#

# bad_packets chain
#

# Drop INVALID packets immediately
$IPT -A bad_packets -p ALL -m state --state INVALID -j DROP

# Then check the tcp packets for additional problems
$IPT -A bad_packets -p tcp -j bad_tcp_packets

# All good, so return
$IPT -A bad_packets -p ALL -j RETURN

# bad_tcp_packets chain
#
# All tcp packets will traverse this chain.
# Every new connection attempt should begin with
# a syn packet.  If it doesn't, it is likely a
# port scan.  This drops packets in state
# NEW that are not flagged as syn packets.

$IPT -A bad_tcp_packets -p tcp ! --syn -m state --state NEW -j LOG \
    --log-prefix "New not syn: "
$IPT -A bad_tcp_packets -p tcp ! --syn -m state --state NEW -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL NONE -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL NONE -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL ALL -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL ALL -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL FIN,URG,PSH -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL FIN,URG,PSH -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL SYN,RST,ACK,FIN,URG -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags ALL SYN,RST,ACK,FIN,URG -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags SYN,RST SYN,RST -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags SYN,RST SYN,RST -j DROP

$IPT -A bad_tcp_packets -p tcp --tcp-flags SYN,FIN SYN,FIN -j LOG \
    --log-prefix "Stealth scan: "
$IPT -A bad_tcp_packets -p tcp --tcp-flags SYN,FIN SYN,FIN -j DROP

# All good, so return
$IPT -A bad_tcp_packets -p tcp -j RETURN

# icmp_packets chain
$IPT -A icmp_packets --fragment -p ICMP -j DROP
$IPT -A icmp_packets -p ICMP -s 0/0 --icmp-type 8 -j REJECT
# Time Exceeded
$IPT -A icmp_packets -p ICMP -s 0/0 --icmp-type 11 -j ACCEPT
# Not matched, so return so it will be logged
$IPT -A icmp_packets -p ICMP -j RETURN

# TCP & UDP
# udp_inbound chain
$IPT -A udp_inbound -p UDP -s 0/0 --destination-port 137 -j DROP
$IPT -A udp_inbound -p UDP -s 0/0 --destination-port 138 -j DROP
# Not matched, so return for logging
$IPT -A udp_inbound -p UDP -j RETURN

# udp_outbound chain
# No match, so ACCEPT
$IPT -A udp_outbound -p UDP -s 0/0 -j ACCEPT

# tcp_inbound chain
# HTTP
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 80 -j ACCEPT

# HTTPS (Secure Web Server)
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 443 -j ACCEPT

# jabber Server
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 5280 -j ACCEPT

# sshd
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 22 -j ACCEPT

# Email Server (SMTP)
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 25 -j ACCEPT

# Nagios Server
$IPT -A tcp_inbound -p TCP -s 0/0 --destination-port 5666 -j ACCEPT

# Not matched, so return so it will be logged
$IPT -A tcp_inbound -p TCP -j RETURN

# tcp_outbound chain
# No match, so ACCEPT
$IPT -A tcp_outbound -p TCP -s 0/0 -j ACCEPT

###############################################################################
#
# INPUT Chain
#

echo "Process INPUT chain ..."

# Allow all on localhost interface
$IPT -A INPUT -p ALL -i $LO_IFACE -j ACCEPT

# Drop bad packets
$IPT -A INPUT -p ALL -j bad_packets
$IPT -A INPUT -p ALL -d 224.0.0.1 -j DROP

# Inbound Internet Packet Rules
# Accept VPN tunnel connection without limitation
if [ "$VPN_ENABLED" = "true" ]
then
    $IPT -A INPUT -p UDP -s 0/0 --destination-port 1194 -j ACCEPT
    $IPT -A INPUT -p ALL -i tun+ -j ACCEPT
    $IPT -A INPUT -p ALL -i tap+ -j ACCEPT
    $IPT -A INPUT -p ALL -i ppp+ -j ACCEPT
fi

# Accept Established Connections
$IPT -A INPUT -p ALL $INET_IFACE -m state --state ESTABLISHED,RELATED \
     -j ACCEPT

# Route the rest to the appropriate user chain
$IPT -A INPUT -p TCP $INET_IFACE -j tcp_inbound
$IPT -A INPUT -p UDP $INET_IFACE -j udp_inbound
$IPT -A INPUT -p tcp --dport 21 -j ACCEPT
$IPT -A INPUT -p tcp --dport 50505:50512 -j ACCEPT
$IPT -A INPUT -p ICMP $INET_IFACE -j icmp_packets

# Drop without logging broadcasts that get this far.
$IPT -A INPUT -m pkttype --pkt-type broadcast -j DROP

###############################################################################
#
# FORWARD Chain
#

echo "Process FORWARD chain ..."

# Used if forwarding for a private network


###############################################################################
#
# OUTPUT Chain
#

echo "Process OUTPUT chain ..."

# Generally trust the firewall on output

# However, invalid icmp packets need to be dropped
$IPT -A OUTPUT -m state -p icmp --state INVALID -j DROP

# Localhost
$IPT -A OUTPUT -p ALL -s $LO_IP -j ACCEPT
$IPT -A OUTPUT -p ALL -o $LO_IFACE -j ACCEPT

#VPN
if [ "$VPN_ENABLED" = "true" ]
then
    $IPT -A OUTPUT -p ALL -o tun+ -j ACCEPT
    $IPT -A OUTPUT -p ALL -o tap+ -j ACCEPT
    $IPT -A OUTPUT -p ALL -o ppp+ -j ACCEPT

fi

# To internet
$IPT -A OUTPUT -p ALL $INET_IFACE_O -j ACCEPT

###############################################################################
#
# nat table
#
###############################################################################

echo "Load rules for nat table ..."

###############################################################################
#
# PREROUTING chain
#


###############################################################################
#
# POSTROUTING chain
#


###############################################################################
#
# mangle table
#

echo "Load rules for mangle table ..."


###############################################################################
#
# ipv6
#

echo "Load rules for ipv6"

# ipv6 tables
IPsixT=`which ip6tables`

$IPsixT -F || echo "Error ipv6 flush"

# source = localhost erlauben
$IPsixT -I INPUT -s ::1 -d ::1 -j ACCEPT || echo "Error ipv6 lo"

# HTTPS (Secure Web Server)
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 80 -j ACCEPT || echo "Error ipv6 80"

# HTTPS (Secure Web Server)
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 443 -j ACCEPT || echo "Error ipv6 443"

# jabber Server
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 5280 -j ACCEPT || echo "Error ipv6 5280"

# sshd
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 22 -j ACCEPT || echo "Error ipv6 22"

# Email Server (SMTP)
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 25 -j ACCEPT || echo "Error ipv6 25"

# Nagios Server
$IPsixT -A INPUT -p TCP -s 0/0 --destination-port 5666 -j ACCEPT || echo "Error ipv6 5666"

$IPsixT -P INPUT DROP || echo "Error ipv6 dropp all"

###############################################################################
#
# docker specific rules
#

$IPT -I tcp_inbound -i docker0 -p tcp --dport 8080 -j ACCEPT

###############################################################################
#
# JUST specific rules (auto generated from <juco_home>/bin/custom_firewall_rules)
#
###############################################################################
